<?php
/**
 * Copyright (c) Heinlein Professional Linux Support GmbH
 * 
 * created   : 28.07.2010
 * created by: Christoph Graupner <c.graupner@heinlein-support.de>
 *
 * last changed   : $LastChangedDate: 2011-12-28 17:05:55 +0100 (Mi, 28. Dez 2011) $
 * last changed by: $LastChangedBy: cgr $
 * revision       : $LastChangedRevision: 9196 $
 * SVN head URL   : $HeadURL: https://buero-svn.jpberlin.de/Dev-Repository/AC/GUI/Core/trunk/hpls.ac.core2.gui.common/Hpls/Cli/Getopt.php $
 * 
 * author         : Christoph Graupner <c.graupner@heinlein-support.de>
 */
/***/
require_once 'Zend/Log.php';
require_once 'Zend/Registry.php';
require_once 'Zend/Console/Getopt.php';
require_once 'Hpls/Exception/Cli/Getopt.php';

/**
 * Erweiterung der Zend_Getopt-Parser-Klasse um folgende Features:
 * <ul>
 * <li>mehrfache Optionen mit Argumenten werden als Argument-Array gespeichert, statt dass eine Exception geworfen wird oder
 * das letzte Argument alle vorherigen überschreibt.</li>
 * <li>Ein zweites '=' bei einer Long-option wird dem option-Argument zugeschlagen, statt es zu entfernen und den Rest unter den Tisch fallen zu lassen.</li>
 * <li>Definiert ein eigenes rule-schema 'hpls'</li>
 * </ul>
 * @todo comment
 *
 * @author Christoph Graupner <c.graupner@heinlein-support.de>
 * @copyright Copyright (c) Heinlein Professional Linux Support GmbH
 *
 * @package AC
 * @subpackage CLI-UI
 * @version $LastChangedRevision: 9196 $
 * @since 1.0.0.5
 */
class Hpls_Cli_Getopt extends Zend_Console_Getopt {
	const MODE_HPLS = 'hpls';
	
	protected $_log = NULL;

	/**
	 * Parse command-line arguments for a single long option.
	 * A long option is preceded by a double '--' character.
	 * Long options may not be clustered.
	 *
	 * @param  mixed &$argv
	 * @return void
	 */
	protected function _parseLongOption(&$argv) {
		$optionWithParam = ltrim(array_shift($argv), '-');
		$l = explode('=', $optionWithParam);
		$flag = array_shift($l);
		//fehlerkorrektur, wenn 2x '=' vorkam -> zweites gehört zum Parameter
		if (count($l) > 1) {
			$param = implode('=', $l);
		} else {
			$param = array_shift($l);
		}
		if (isset($param)) {
			array_unshift($argv, $param);
		}
		$this->_parseSingleOption($flag, $argv);
	}

	/**
	 * Parse command-line arguments for a single option.
	 *
	 * @param  string $flag
	 * @param  mixed  $argv
	 * @throws Zend_Console_Getopt_Exception
	 * @return void
	 */
	protected function _parseSingleOption($flag, &$argv) {
		if ($this->_getoptConfig[self::CONFIG_IGNORECASE]) {
			$flag = strtolower($flag);
		}
		if (!isset($this->_ruleMap[$flag])) {
			/**
			 * @see Zend_Console_Getopt_Exception
			 */
			throw new Zend_Console_Getopt_Exception("Option \"$flag\" is not recognized.", $this->getUsageMessage());
		}
		$realFlag = $this->_ruleMap[$flag];
		switch ($this->_rules[$realFlag]['param']) {
			case 'required' :
				if (count($argv) > 0) {
					$param = array_shift($argv);
					$this->_checkParameterType($realFlag, $param);
				} else {
					/**
					 * @see Zend_Console_Getopt_Exception
					 */
					throw new Zend_Console_Getopt_Exception("Option \"$flag\" requires a parameter.", $this->getUsageMessage());
				}
				break;
			case 'optional' :
				if (count($argv) > 0 && substr($argv[0], 0, 1) != '-') {
					$param = array_shift($argv);
					$this->_checkParameterType($realFlag, $param);
				} else {
					$param = true;
				}
				break;
			default :
				$param = true;
		}
		
		//abweichend von Zend: mehrfaches vorkommen -> in array statt überschreiben
		if (isset($this->_options[$realFlag])) {
			$old = $this->_options[$realFlag];
			if (is_array($old) === FALSE) {
				$old = array(
						$old );
			}
			$old[] = $param;
			$this->_options[$realFlag] = $old;
		} else {
			$this->_options[$realFlag] = $param;
		}
	}

	/**
	 * Zend-style format mit folgender Abwandlung:
	 * <ul>
	 * <li>Optionsnamen dürfen doppelt vorkommen, aber müssen den gleichen Parametertyp haben (required|optional, none|string|word|integer)
	 * </li>
	 * </ul>
	 *
	 * @param  array $rules
	 * @throws Zend_Console_Getopt_Exception
	 * @return void
	 */
	protected function _addRulesModeHpls($rules) {
		$this->log()->info(__METHOD__);
		foreach ($rules as $ruleCode => $helpMessage) {
			$tokens = preg_split('/([=-])/', $ruleCode, 2, PREG_SPLIT_DELIM_CAPTURE);
			$flagList = array_shift($tokens);
			$delimiter = array_shift($tokens);
			$paramType = array_shift($tokens);
			if ($this->_getoptConfig[self::CONFIG_IGNORECASE]) {
				$flagList = strtolower($flagList);
			}
			$flags = explode('|', $flagList);
			$rule = array();
			$mainFlag = $flags[0];
			
			//gehören alle aliase zum selben original? 
			$found = FALSE;
			foreach ($flags as $flag) {
				if (array_key_exists($flag, $this->_ruleMap)) {
					$main = $this->_ruleMap[$flag];
					if ($found !== FALSE && strcmp($main, $found) !== 0) {
						throw new Hpls_Exception_Cli_Getopt("Option \"$found\" is being defined more than once by \"$flag\" in variing ways.");
					}
					$found = $flag;
				}
			}
			
			if ($found !== FALSE) {
				$mainFlag = $this->_ruleMap[$found];
				$rule = $this->_rules[$mainFlag];
			}
			
			foreach ($flags as $flag) {
				if (empty($flag)) {
					/**
					 * @see Zend_Console_Getopt_Exception
					 */
					throw new Hpls_Exception_Cli_Getopt("Blank flag not allowed in rule \"$ruleCode\".");
				}
				if (strlen($flag) == 1) {
					if (isset($this->_ruleMap[$flag])) {
						//alias schon vorhanden, nix tun, parameterprüfung kommt weiter unten
						if (strcmp($this->_ruleMap[$flag], $mainFlag) !== 0) {
							throw new Hpls_Exception_Cli_Getopt("Option alias \"$flag\" will link to at least 2 different originals: '" . $this->_ruleMap[$flag] . "' and '$mainFlag'.");
						}
					
		//                        /**
					//                         * @see Zend_Console_Getopt_Exception
					//                         */
					//                        throw new Hpls_Exception_Cli_Getopt(
					//                            "Option \"-$flag\" is being defined more than once.");
					} else {
						
						$this->_ruleMap[$flag] = $mainFlag;
						$rule['alias'][] = $flag;
					}
				} else {
					if (isset($this->_rules[$flag]) || isset($this->_ruleMap[$flag])) {
						//alias schon vorhanden, nix tun, parameterprüfung kommt weiter unten
					//                    	/**
					//                         * @see Zend_Console_Getopt_Exception
					//                         */
					//                        throw new Hpls_Exception_Cli_Getopt(
					//                            "Option \"--$flag\" is being defined more than once.");
					} else {
						$this->_ruleMap[$flag] = $mainFlag;
						$rule['alias'][] = $flag;
					}
				}
			}
			$param = array();
			if (isset($delimiter)) {
				switch ($delimiter) {
					case self::PARAM_REQUIRED :
						$param['param'] = 'required';
						break;
					case self::PARAM_OPTIONAL :
					default :
						$param['param'] = 'optional';
				}
				switch (substr($paramType, 0, 1)) {
					case self::TYPE_WORD :
						$param['paramType'] = 'word';
						break;
					case self::TYPE_INTEGER :
						$param['paramType'] = 'integer';
						break;
					case self::TYPE_STRING :
					default :
						$param['paramType'] = 'string';
				}
			} else {
				$param['param'] = 'none';
			}
			//existiert 'param' -> dann existierte rule -> dann prüfen ob gleicher paramtyp
			if (isset($rule['param'])) {
				if (strcmp($rule['param'], $param['param']) !== 0) {
					//ungleiche parametertypen -> exit
					throw new Hpls_Exception_Cli_Getopt("Parameter definitions are not compatible while defining '$mainFlag' (" . $rule['param'] . ") again by '$flag' (" . $param['param'] . ").");
				}
				if (strcmp($rule['paramType'], $param['paramType']) !== 0) {
					throw new Hpls_Exception_Cli_Getopt("Parameter definitions are not compatible while defining '$mainFlag' (" . $rule['paramType'] . ") again by '$flag' (" . $param['paramType'] . ").");
				}
			}
			$rule['param'] = $param['param'];
			if (isset($param['paramType'])) {
				$rule['paramType'] = $param['paramType'];
			}
			
			$rule['help'] = $helpMessage;
			$this->_rules[$mainFlag] = $rule;
		}
	}

	public static function splitUp($optName) {
		$ret = array();
		$tokens = preg_split('/([=-])/', $optName, 2, PREG_SPLIT_DELIM_CAPTURE);
		$ret['names'] = explode('|', array_shift($tokens));
		$ret['delimiter'] = array_shift($tokens);
		$ret['paramType'] = array_shift($tokens);
		return $ret;
	
	}

	public static function getParamType(array $splitUpList) {
		
		$param = array();
		if (isset($splitUpList['delimiter']) && !empty($splitUpList['delimiter'])) {
			switch ($splitUpList['delimiter']) {
				case self::PARAM_REQUIRED :
					$param['param'] = 'required';
					break;
				case self::PARAM_OPTIONAL :
				default :
					$param['param'] = 'optional';
			}
			switch (substr($splitUpList['paramType'], 0, 1)) {
				case self::TYPE_WORD :
					$param['paramType'] = 'word';
					break;
				case self::TYPE_INTEGER :
					$param['paramType'] = 'integer';
					break;
				case self::TYPE_STRING :
				default :
					$param['paramType'] = 'string';
			}
		} else {
			$param['param'] = 'none';
		}
		return $param;
	}

	/**
	 * @return Zend_Log
	 */
	protected function log() {
		if (is_null($this->_log)) {
			if (Zend_Registry::isRegistered('logger')) {
				$this->_log = Zend_Registry::get('logger');
			} else {
				require_once 'Zend/Log/Writer/Null.php';
				$this->_log = new Zend_Log(new Zend_Log_Writer_Null());
			}
		}
		return $this->_log;
	}
}